/*
QUERY: select floor(deweyClass) as deweyCl,HOUR(cout) as hour, count(*) from inraw where deweyClass != 'null'
and month(cout) = '04' and date(cout) != '1970-01-01' group by deweyCl order by deweyCl;
*/


// The scale of our world
float zoom;
// A vector to store the offset from the center
PVector offset;
// The previous offset
PVector poffset;
// A vector for the mouse position
PVector mouse;

PFont font1 = createFont("Helvetica", 15); 
PFont font2 = createFont("Helvetica", 30);
int border = 20;
int[][] hourCoutByDewey = new int[1000][24];  //[dewey][hour]
int[][] tempArray = new int[1000][24];  //[dewey][hour]
int displayMode = 0;

boolean showGrid = false;
boolean showMousePosition = false;
int padding = 10;
int boxSize = 12;

float xScalar = 0.65; 
float yScalar = 30;  
float labelSpacing = yScalar;
float mouseYScalar = 0.4;
float mouseXScalar = 1.515;
float mouseXOffset = 118.6;
float alphaScalar = 0.3;
float offsetX = -320;
float offsetY = -280;


float horizontalGridDivisions = 100;
float verticalGridDivisions = 5;

float horizontalSquareSize;
float verticalSquareSize;

void setup() {
  size(800, 600);
  zoom = 1.0;
  offset = new PVector(0, 0);
  poffset = new PVector(0, 0);

  horizontalSquareSize = (width-136.5)/horizontalGridDivisions;
  verticalSquareSize = height/verticalGridDivisions;
  
  colorMode(HSB,360,100,100); //color mode- like RGB or HSB (learn to use HSB because it's the best: http://en.wikipedia.org/wiki/HSL_and_HSV)
  smooth();

  //import the data from our text file 
  String savedData[] = loadStrings("savedData.txt");
  println("QUERY: " + savedData[0]);

  int curDewey = 1;
  int oldDewey = 1;
  int curHour = 0;

  for (int i=1; i < savedData.length; i++) {
    String temp[] = split(savedData[i], ",");
    curDewey = Integer.parseInt(temp[0]);
    curHour = Integer.parseInt(temp[1]);
    tempArray[curDewey][curHour]=Integer.parseInt(temp[2]);
    hourCoutByDewey[curDewey][curHour]=hourCoutByDewey[curDewey][curHour]+tempArray[curDewey][curHour]; // sum values    
  }
  /*
  //test
for (int i=1; i < 1000; i++) {
    println(hourCoutByDewey[i][12]); // sum values    
//    println(hourCoutByDewey[i][9]); // sum values    
  }  
  */
}

void draw() {
    int rectOffset = boxSize/2;
    background(359);
       pushMatrix();
    if(showGrid == true){
      drawGrid();
    }
    if(showMousePosition == true){
      drawMouse();
    }
    
    // Everything must be drawn relative to center
    translate(width/2, height/2);
    // Use scale for 2D "zoom"
    scale(zoom);
    // The offset (note how we scale according to the zoom)
    translate(offset.x/zoom, offset.y/zoom);
  
    //draw transactions
    strokeWeight(1.0);
    strokeCap(SQUARE);
    fill(220,100,150,150);  
    for(int i = 0;i<1000;i++){  // loop through dewey classes
      for(int j = 0;j<24;j++){  // loop through dewey classes
        //float tempX = (i*(width-horizontalSquareSize)/100)+padding; 
        if(hourCoutByDewey[i][j]!=0) {
          stroke(100,0,200,hourCoutByDewey[i][j]*alphaScalar);
          rect(i*xScalar+offsetX,j*yScalar+offsetY-108,boxSize,boxSize);
        //rect(height-padding-hourCoutByDewey[i][j]*scalar, boxSize, boxSize); 
        //line(tempX, height-padding, tempX, height-padding-hourCoutByDewey[i]*scalar);
        }
      }
    }
    
    //draw vertical labels
    textAlign(RIGHT,CENTER);
    textSize(11);

    fill(200,125,20, 140);  

    for(int i = 0;i<21;i++){
        if(i>6){
           if(i<=11){
             text("a.m.", -340, (i*labelSpacing)+offsetY-105);       
           }
           if(i>11) {
              text("p.m.", -340, (i*labelSpacing)+offsetY-105);       
           }
           if(i>12) {
             text(i-12, -372, (i*labelSpacing)+offsetY-105);
           }
           else if(i<20) {
             text(i, -372, (i*labelSpacing)+offsetY-105);   
           }
        }
    }

     
    popMatrix();
  
  // Draw text not panned / zoomed
    fill(200,125,20, 140);  
    textAlign(LEFT,TOP);
    textSize(14);
    text("Seattle Public Library",padding,padding);
    text("Transactions by Dewey Category",padding,padding+20);
    text("April 12, 2011",padding,padding+40);
    text("* Click to see Dewey number",padding,padding+60);
}

void drawMouse(){
  /*
    // Everything must be drawn relative to center
    translate(width/2, height/2);
  
    // Use scale for 2D "zoom"
    scale(zoom);
    // The offset (note how we scale according to the zoom)
    translate(offset.x/zoom, offset.y/zoom);

  */
  
  float cursorX = mouse.x-offset.x;
  float cursorY = mouse.y-offset.y;
    
  stroke(0,0,70);
  strokeWeight(1.0);

  if( (cursorX*mouseXScalar-mouseXOffset)>0 && (cursorX*mouseXScalar-mouseXOffset) < 1000)
  {
    line(mouse.x,height,mouse.x,0);
    textAlign(LEFT,BOTTOM);
    textFont(font1);
    
/*   println("offset.y:");
     println(offset.y);
*/    
    if((cursorX*mouseXScalar-mouseXOffset)>0){                // fix this
      text(round(cursorX*mouseXScalar-mouseXOffset),cursorX-60, padding+100); 
    }
  }
}

void drawGrid(){
  //draw horizontal grid
  stroke(0,0,80);
  strokeWeight(2);
  line(horizontalSquareSize+74,110,horizontalSquareSize+74, height-70);
  for(int i = 2;i<=horizontalGridDivisions;i++){
    if(i % 10 == 0)
    {
      stroke(0,0,80);
      strokeWeight(2);
      line(i*horizontalSquareSize+74,110,i*horizontalSquareSize+74, height-70);
    }
    else
    {
     stroke(0,0,10);
     strokeWeight(0.1);
      line(i*horizontalSquareSize+74,110,i*horizontalSquareSize+74, height-70);
    }
  }
  //draw vertical grid
  stroke(0,0,80);
  strokeWeight(0.5);
  for(int i = 0;i<=verticalGridDivisions-1;i++){
    line(padding,i*verticalSquareSize-padding,width-50, i*verticalSquareSize-padding);
//    line(0,i*verticalSquareSize+padding,width, i*verticalSquareSize+padding);
  }
}

void keyPressed(){
 //press spacebar to toggle the grid on and off  
 if(key == ' '){
   if(showGrid == true){
     showGrid = false;
   }
   else{showGrid = true;}
 } 
}

void mousePressed(){
  // Store the mouse and the previous offset

  mouse = new PVector(mouseX, mouseY);
  poffset.set(offset);
 
  if(showMousePosition == true){
     showMousePosition = false;
   }
   else{showMousePosition = true;}
   
}


// Calculate the new offset based on change in mouse vs. previous offsey
void mouseDragged() {
//  offset.x = mouseX - mouse.x + poffset.x;
//  offset.y = mouseY - mouse.y + poffset.y;
}

